from datetime import date, datetime

from .time import at_midnight, date_after


def test_at_midnight():
    assert at_midnight(date(1989, 2, 2)) == datetime(1989, 2, 2, 0, 0, 0)


def test_date_after():
    day = date(1999, 12, 31)
    assert date_after(day, 3) == date(2000, 1, 3)



from datetime import date

from freezegun import freeze_time

from ..abstract.time_filter import TimeFilter
from .client import DatabricksClient, _day_hour_to_epoch_ms


def test__day_hour_to_epoch_ms():
    _day_hour_to_epoch_ms(date(2023, 2, 14), 14) == 1644847200000


@freeze_time("2023-7-4")
def test_DatabricksClient__time_filter():
    # default is yesterday
    default_time_filter = None
    default_filter = DatabricksClient._time_filter(default_time_filter)
    expected_default = {
        "filter_by": {
            "query_start_time_range": {
                "end_time_ms": 1688428800000,  # July 4, 2023 12:00:00 AM GMT
                "start_time_ms": 1688342400000,  # July 3, 2023 12:00:00 AM GMT
            }
        }
    }
    assert default_filter == expected_default

    # custom time (from execution_date in DAG for example)
    time_filter = TimeFilter(day=date(2020, 10, 15))
    custom_filter = DatabricksClient._time_filter(time_filter)
    expected_custom = {
        "filter_by": {
            "query_start_time_range": {
                "end_time_ms": 1602806400000,  # October 16, 2020 12:00:00 AM
                "start_time_ms": 1602720000000,  # October 15, 2020 12:00:00 AM
            }
        }
    }
    assert custom_filter == expected_custom

    # hourly extraction: note that hour_min == hour_max
    hourly = TimeFilter(day=date(2023, 4, 14), hour_min=4, hour_max=4)
    hourly_filter = DatabricksClient._time_filter(hourly)
    expected_hourly = {
        "filter_by": {
            "query_start_time_range": {
                "end_time_ms": 1681448400000,  # April 14, 2023 5:00:00 AM
                "start_time_ms": 1681444800000,  # April 14, 2023 4:00:00 AM
            }
        }
    }
    assert hourly_filter == expected_hourly


class MockDatabricksClient(DatabricksClient):
    def __init__(self):
        self._db_allowed = ["prd", "staging"]
        self._db_blocked = ["dev"]


def test_DatabricksClient__keep_catalog():
    client = MockDatabricksClient()
    assert client._keep_catalog("prd")
    assert client._keep_catalog("staging")
    assert not client._keep_catalog("dev")
    assert not client._keep_catalog("something_unknown")



from dataclasses import field

from pydantic.dataclasses import dataclass

from ...utils import from_env

_HOST = "CASTOR_DATABRICKS_HOST"
_TOKEN = "CASTOR_DATABRICKS_TOKEN"  # noqa: S105


@dataclass
class DatabricksCredentials:
    """
    Credentials needed by Databricks client
    Requires:
    - host
    - token
    """

    host: str
    token: str = field(metadata={"sensitive": True})


def to_credentials(params: dict) -> DatabricksCredentials:
    """extract Databricks credentials"""
    host = params.get("host") or from_env(_HOST)
    token = params.get("token") or from_env(_TOKEN)
    return DatabricksCredentials(
        host=host,
        token=token,
    )



from pydantic import BaseSettings, Extra, Field

_DEFAULT_CLOUD_URL = "https://cloud.getdbt.com"


class DbtCredentials(BaseSettings):
    """dbt credentials: host has default value"""

    host: str = Field(default=_DEFAULT_CLOUD_URL, env="CASTOR_DBT_HOST")
    job_id: str = Field(..., env="CASTOR_DBT_JOB_ID")
    token: str = Field(..., env="CASTOR_DBT_TOKEN")

    class Config:
        """constructor settings: ignore extra kwargs provided"""

        extra = Extra.ignore



import json
from typing import Optional

import requests

from ...warehouse.databricks.credentials import DatabricksCredentials

DEFAULT_TIMEOUT_MS = 30_000
APICredentials = DatabricksCredentials


class APIClient:
    """
    API client
    - used for Databricks Unity Catalog for now
    - authentication via access token for now
    """

    def __init__(self, credentials: APICredentials):
        self.credentials = credentials
        self._timeout = DEFAULT_TIMEOUT_MS

    @staticmethod
    def build_url(host: str, path: str):
        if not host.startswith("https://"):
            host = "https://" + host
        return f"{host.strip('/')}/{path}"

    def _headers(self):
        return {
            "Content-type": "application/json",
            "Authorization": f"Bearer {self.credentials.token}",
        }

    def get(self, path: str, payload: Optional[dict] = None) -> dict:
        """
        path: REST API operation path, such as /api/2.0/clusters/get
        """
        url = self.build_url(self.credentials.host, path)
        response = requests.get(
            url,
            data=json.dumps(payload or dict()),
            headers=self._headers(),
            timeout=self._timeout,
        )

        if response.content:
            return json.loads(response.content)

        return {}
